@@ -27,15 +27,16 @@ build_time_vars = {'ABIFLAGS': 'm',
  'CC': 'gcc -pthread',
  'CCSHARED': '-fPIC',
  'CFLAGS': '-Wno-unused-result -Wsign-compare -DNDEBUG -g -fwrapv -O3 -Wall '
-           '-Wstrict-prototypes -march=armv8-a -O2 -pipe -fstack-protector '
-           '--param=ssp-buffer-size=4 -march=armv8-a -O2 -pipe '
-           '-fstack-protector --param=ssp-buffer-size=4 -march=armv8-a -O2 '
-           '-pipe -fstack-protector --param=ssp-buffer-size=4',
+           '-Wstrict-prototypes -march=armv8-a -O2 -pipe '
+           '-fstack-protector-strong --param=ssp-buffer-size=4 -march=armv8-a '
+           '-O2 -pipe -fstack-protector-strong --param=ssp-buffer-size=4 '
+           '-march=armv8-a -O2 -pipe -fstack-protector-strong '
+           '--param=ssp-buffer-size=4',
  'CFLAGSFORSHARED': '-fPIC',
  'CFLAGS_NODIST': '',
  'CONFIGFILES': 'configure configure.ac acconfig.h pyconfig.h.in '
                 'Makefile.pre.in',
- 'CONFIGURE_CFLAGS': '-march=armv8-a -O2 -pipe -fstack-protector '
+ 'CONFIGURE_CFLAGS': '-march=armv8-a -O2 -pipe -fstack-protector-strong '
                      '--param=ssp-buffer-size=4',
  'CONFIGURE_CFLAGS_NODIST': '-std=c99 -Wextra -Wno-unused-result '
                             '-Wno-unused-parameter '
@@ -48,15 +49,15 @@ build_time_vars = {'ABIFLAGS': 'm',
                 "'--with-dbmliborder=gdbm:ndbm' '--with-system-ffi' "
                 "'--with-system-libmpdec' "
                 "'--enable-loadable-sqlite-extensions' '--without-ensurepip' "
-                "'CFLAGS=-march=armv8-a -O2 -pipe -fstack-protector "
+                "'CFLAGS=-march=armv8-a -O2 -pipe -fstack-protector-strong "
                 "--param=ssp-buffer-size=4' "
                 "'LDFLAGS=-Wl,-O1,--sort-common,--as-needed,-z,relro' "
                 "'CPPFLAGS=-D_FORTIFY_SOURCE=2'",
  'CONFINCLUDEDIR': '/usr/include',
  'CONFINCLUDEPY': '/usr/include/python3.6m',
  'COREPYTHONPATH': ':',
- 'COVERAGE_INFO': '/build/python/src/Python-3.6.0/coverage.info',
- 'COVERAGE_REPORT': '/build/python/src/Python-3.6.0/lcov-report',
+ 'COVERAGE_INFO': '/build/python/src/Python-3.6.1/coverage.info',
+ 'COVERAGE_REPORT': '/build/python/src/Python-3.6.1/lcov-report',
  'COVERAGE_REPORT_OPTIONS': '--no-branch-coverage --title "CPython lcov '
                             'report"',
  'CPPFLAGS': '-I. -I./Include -D_FORTIFY_SOURCE=2 -D_FORTIFY_SOURCE=2',
@@ -93,6 +94,9 @@ build_time_vars = {'ABIFLAGS': 'm',
  'FLOCK_NEEDS_LIBBSD': 0,
  'GETPGRP_HAVE_ARG': 0,
  'GETTIMEOFDAY_NO_TZ': 0,
+ 'GITBRANCH': '',
+ 'GITTAG': '',
+ 'GITVERSION': '',
  'GNULD': 'yes',
  'GRAMMAR_C': 'Python/graminit.c',
  'GRAMMAR_H': 'Include/graminit.h',
@@ -202,7 +206,7 @@ build_time_vars = {'ABIFLAGS': 'm',
  'HAVE_GCC_UINT128_T': 1,
  'HAVE_GETADDRINFO': 1,
  'HAVE_GETC_UNLOCKED': 1,
- 'HAVE_GETENTROPY': 0,
+ 'HAVE_GETENTROPY': 1,
  'HAVE_GETGROUPLIST': 1,
  'HAVE_GETGROUPS': 1,
  'HAVE_GETHOSTBYNAME': 0,
@@ -221,7 +225,7 @@ build_time_vars = {'ABIFLAGS': 'm',
  'HAVE_GETPID': 1,
  'HAVE_GETPRIORITY': 1,
  'HAVE_GETPWENT': 1,
- 'HAVE_GETRANDOM': 0,
+ 'HAVE_GETRANDOM': 1,
  'HAVE_GETRANDOM_SYSCALL': 1,
  'HAVE_GETRESGID': 1,
  'HAVE_GETRESUID': 1,
@@ -421,6 +425,7 @@ build_time_vars = {'ABIFLAGS': 'm',
  'HAVE_SYS_NDIR_H': 0,
  'HAVE_SYS_PARAM_H': 1,
  'HAVE_SYS_POLL_H': 1,
+ 'HAVE_SYS_RANDOM_H': 1,
  'HAVE_SYS_RESOURCE_H': 1,
  'HAVE_SYS_SELECT_H': 1,
  'HAVE_SYS_SENDFILE_H': 1,
@@ -475,9 +480,6 @@ build_time_vars = {'ABIFLAGS': 'm',
  'HAVE_WRITEV': 1,
  'HAVE_ZLIB_COPY': 1,
  'HAVE__GETPTY': 0,
- 'HGBRANCH': '',
- 'HGTAG': '',
- 'HGVERSION': '',
  'HOST_GNU_TYPE': 'aarch64-unknown-linux-gnu',
  'INCLDIRSTOMAKE': '/usr/include /usr/include /usr/include/python3.6m '
                    '/usr/include/python3.6m',
@@ -599,19 +601,20 @@ build_time_vars = {'ABIFLAGS': 'm',
  'PYTHON_HEADERS': '\\',
  'PYTHON_OBJS': '\\',
  'PY_CFLAGS': '-Wno-unused-result -Wsign-compare -DNDEBUG -g -fwrapv -O3 -Wall '
-              '-Wstrict-prototypes -march=armv8-a -O2 -pipe -fstack-protector '
+              '-Wstrict-prototypes -march=armv8-a -O2 -pipe '
+              '-fstack-protector-strong --param=ssp-buffer-size=4 '
+              '-march=armv8-a -O2 -pipe -fstack-protector-strong '
               '--param=ssp-buffer-size=4 -march=armv8-a -O2 -pipe '
-              '-fstack-protector --param=ssp-buffer-size=4 -march=armv8-a -O2 '
-              '-pipe -fstack-protector --param=ssp-buffer-size=4',
+              '-fstack-protector-strong --param=ssp-buffer-size=4',
  'PY_CFLAGS_NODIST': '-std=c99 -Wextra -Wno-unused-result '
                      '-Wno-unused-parameter -Wno-missing-field-initializers',
  'PY_CORE_CFLAGS': '-Wno-unused-result -Wsign-compare -DNDEBUG -g -fwrapv -O3 '
                    '-Wall -Wstrict-prototypes -march=armv8-a -O2 -pipe '
-                   '-fstack-protector --param=ssp-buffer-size=4 -march=armv8-a '
-                   '-O2 -pipe -fstack-protector --param=ssp-buffer-size=4 '
-                   '-march=armv8-a -O2 -pipe -fstack-protector '
-                   '--param=ssp-buffer-size=4 -std=c99 -Wextra '
-                   '-Wno-unused-result -Wno-unused-parameter '
+                   '-fstack-protector-strong --param=ssp-buffer-size=4 '
+                   '-march=armv8-a -O2 -pipe -fstack-protector-strong '
+                   '--param=ssp-buffer-size=4 -march=armv8-a -O2 -pipe '
+                   '-fstack-protector-strong --param=ssp-buffer-size=4 '
+                   '-std=c99 -Wextra -Wno-unused-result -Wno-unused-parameter '
                    '-Wno-missing-field-initializers -I. -I./Include '
                    '-D_FORTIFY_SOURCE=2 -D_FORTIFY_SOURCE=2 -fPIC '
                    '-DPy_BUILD_CORE',
@@ -627,7 +630,7 @@ build_time_vars = {'ABIFLAGS': 'm',
  'READELF': 'readelf',
  'RESSRCDIR': 'Mac/Resources/framework',
  'RETSIGTYPE': 'void',
- 'RUNSHARED': 'LD_LIBRARY_PATH=/build/python/src/Python-3.6.0',
+ 'RUNSHARED': 'LD_LIBRARY_PATH=/build/python/src/Python-3.6.1',
  'SCRIPTDIR': '/usr/lib',
  'SETPGRP_HAVE_ARG': 0,
  'SGI_ABI': '',
@@ -668,9 +671,9 @@ build_time_vars = {'ABIFLAGS': 'm',
  'TCLTK_LIBS': '',
  'TESTOPTS': '',
  'TESTPATH': '',
- 'TESTPYTHON': 'LD_LIBRARY_PATH=/build/python/src/Python-3.6.0 ./python',
+ 'TESTPYTHON': 'LD_LIBRARY_PATH=/build/python/src/Python-3.6.1 ./python',
  'TESTPYTHONOPTS': '',
- 'TESTRUNNER': 'LD_LIBRARY_PATH=/build/python/src/Python-3.6.0 ./python '
+ 'TESTRUNNER': 'LD_LIBRARY_PATH=/build/python/src/Python-3.6.1 ./python '
                './Tools/scripts/run_tests.py',
  'TESTTIMEOUT': 1200,
  'THREADOBJ': 'Python/thread.o',
@@ -694,8 +697,8 @@ build_time_vars = {'ABIFLAGS': 'm',
  'WITH_VALGRIND': 0,
  'X87_DOUBLE_ROUNDING': 0,
  'XMLLIBSUBDIRS': 'xml xml/dom xml/etree xml/parsers xml/sax',
- 'abs_builddir': '/build/python/src/Python-3.6.0',
- 'abs_srcdir': '/build/python/src/Python-3.6.0',
+ 'abs_builddir': '/build/python/src/Python-3.6.1',
+ 'abs_srcdir': '/build/python/src/Python-3.6.1',
  'datarootdir': '/usr/share',
  'exec_prefix': '/usr',
  'prefix': '/usr',
